@@ -21,7 +21,7 @@
  * Nishimura.  Please mail <matumoto@math.keio.ac.jp>, if you're using
  * code from this file in your own programs or libraries.
  * Further information on the Mersenne Twister can be found at
- * http://www.math.keio.ac.jp/~matumoto/emt.html
+ * http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
  * This code was adapted to glib by Sebastian Wilhelmi.
  */
 
@@ -49,6 +49,7 @@
 
 #include "glib.h"
 #include "gthreadinit.h"
+#include "galias.h"
 
 #ifdef G_OS_WIN32
 #include <process.h>		/* For getpid() */
@@ -634,3 +635,6 @@ g_random_set_seed (guint32 seed)
   G_UNLOCK (global_random);
 }
 
+
+#define __G_RAND_C__
+#include "galiasdef.c"
